package com.gic.haoban.manage.service.service.out.impl.hm;

import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.UniqueIdUtils;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.base.api.common.PageResult2;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.dto.StaffClerkRelationDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.dto.hm.HmQrcodeDTO;
import com.gic.haoban.manage.api.dto.hm.HmQrcodeListDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmClerkRelationQDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmQrcodeListQDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmQrcodeQDTO;
import com.gic.haoban.manage.api.service.hm.HmQrcodeApiService;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.pojo.bo.hm.HmQrcodeBO;
import com.gic.haoban.manage.service.service.StaffClerkRelationService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.haoban.manage.service.service.hm.HmClerkRelationService;
import com.gic.haoban.manage.service.service.hm.HmQrcodeService;
import com.gic.wechat.api.dto.qywx.welcome.QywxExternalcontactDTO;
import com.gic.wechat.api.dto.qywx.welcome.QywxExternalcontactResultDTO;
import com.gic.wechat.api.service.qywx.QywxUserApiService;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author mozhu
 * @date 2022/7/5 09:34
 */
@Service
public class HmQrcodeApiServiceImpl implements HmQrcodeApiService {

    private static Logger logger = LoggerFactory.getLogger(HmQrcodeApiServiceImpl.class);

    @Autowired
    private HmQrcodeService hmQrcodeService;
    @Autowired
    private QywxUserApiService qywxUserApiService;
    @Autowired
    private WxEnterpriseService wxEnterpriseService;
    @Autowired
    private Config config;
    @Autowired
    private StaffClerkRelationService staffClerkRelationService;
    @Autowired
    private HmClerkRelationService hmClerkRelationService;
    @Autowired
    private ClerkService clerkService;
    @Autowired
    private StoreService storeService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int add(HmQrcodeQDTO hmQrcodeQDTO) {
        String wxEnterpriseId = hmQrcodeQDTO.getWxEnterpriseId();
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        if (wxEnterpriseDTO == null) {
            logger.error("未查询到微信企业");
            return 0;
        }
        hmQrcodeQDTO.setHmId(UniqueIdUtils.uniqueLong());
        Integer hmType = hmQrcodeQDTO.getHmType();
        Long hmId = hmQrcodeQDTO.getHmId();
        if (hmType == 1) {
            hmQrcodeQDTO.setHmCode("HM01" + hmId);
        } else {
            hmQrcodeQDTO.setHmCode("HM02" + hmId);
        }

        List<String> clerkIdList = setStoreClerk(hmQrcodeQDTO, wxEnterpriseId, hmId, hmType);

        //生成活码
        QywxExternalcontactDTO qywxExternalcontactDTO = new QywxExternalcontactDTO();
        qywxExternalcontactDTO.setType(hmQrcodeQDTO.getHmType());
        qywxExternalcontactDTO.setScene(2);
        qywxExternalcontactDTO.setRemark(hmQrcodeQDTO.getName());
        qywxExternalcontactDTO.setSkipVerify(hmQrcodeQDTO.getPassFlag() == 1);
        qywxExternalcontactDTO.setState("HM" + hmId);
        List<String> userIds = staffClerkRelationService.listWxUserIdByClerkIds(clerkIdList);
        qywxExternalcontactDTO.setUser(userIds);
        QywxExternalcontactResultDTO qywxExternalcontactResultDTO = qywxUserApiService.addContactWay(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid(), qywxExternalcontactDTO);
        if (qywxExternalcontactResultDTO != null) {
            hmQrcodeQDTO.setWxQrcode(qywxExternalcontactResultDTO.getQr_code());
            hmQrcodeQDTO.setWxConfigId(qywxExternalcontactResultDTO.getConfig_id());
        }
        return hmQrcodeService.insert(hmQrcodeQDTO);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public int update(HmQrcodeQDTO hmQrcodeQDTO) {
        String wxEnterpriseId = hmQrcodeQDTO.getWxEnterpriseId();
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        if (wxEnterpriseDTO == null) {
            logger.error("未查询到微信企业");
            return 0;
        }
        Long hmId = hmQrcodeQDTO.getHmId();
        Integer hmType = hmQrcodeQDTO.getHmType();

        List<String> clerkIdList = setStoreClerk(hmQrcodeQDTO, wxEnterpriseId, hmId, hmType);

        //更新活码
        QywxExternalcontactDTO qywxExternalcontactDTO = new QywxExternalcontactDTO();
        qywxExternalcontactDTO.setType(hmQrcodeQDTO.getHmType());
        qywxExternalcontactDTO.setScene(2);
        qywxExternalcontactDTO.setRemark(hmQrcodeQDTO.getName());
        qywxExternalcontactDTO.setSkipVerify(hmQrcodeQDTO.getPassFlag() == 1);
        qywxExternalcontactDTO.setState("HM" + hmId);
        List<String> userIds = staffClerkRelationService.listWxUserIdByClerkIds(clerkIdList);
        qywxExternalcontactDTO.setUser(userIds);
        boolean flag = qywxUserApiService.updateContactWay(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid(), qywxExternalcontactDTO);
        if (!flag) {
            logger.error("更新企微活码失败");
            return 0;
        }
        return hmQrcodeService.update(hmQrcodeQDTO);
    }

    /**
     * 设置门店和导购
     * @param hmQrcodeQDTO
     * @param wxEnterpriseId
     * @param hmId
     * @param hmType
     * @return
     */
    private List<String> setStoreClerk(HmQrcodeQDTO hmQrcodeQDTO, String wxEnterpriseId, Long hmId, Integer hmType) {
        List<String> clerkIdList = hmQrcodeQDTO.getClerkIdList();
        String storeId = hmQrcodeQDTO.getStoreId();
        StoreDTO storeDTO = storeService.getStore(storeId);
        if (storeDTO != null) {
            storeDTO.setStoreCode(storeDTO.getStoreCode());
            storeDTO.setStoreName(storeDTO.getStoreName());
        }
        //多人活码
        if (hmType == 2) {
            insertHmClerkRelation(hmQrcodeQDTO, wxEnterpriseId, hmId, clerkIdList);
        } else {
            String clerkId = clerkIdList.get(0);
            hmQrcodeQDTO.setClerkId(clerkId);
            ClerkDTO clerkDTO = clerkService.getclerkById(clerkId);
            if (clerkDTO != null) {
                hmQrcodeQDTO.setClerkName(clerkDTO.getClerkName());
                hmQrcodeQDTO.setClerkCode(clerkDTO.getClerkCode());
            }
        }
        return clerkIdList;
    }

    /**
     * 多人活码关联关系
     * @param hmQrcodeQDTO
     * @param wxEnterpriseId
     * @param hmId
     * @param clerkIdList
     */
    private void insertHmClerkRelation(HmQrcodeQDTO hmQrcodeQDTO, String wxEnterpriseId, Long hmId, List<String> clerkIdList) {
        List<HmClerkRelationQDTO> clerkRelationQDTOS = new ArrayList<>();
        for (String clerkId : clerkIdList) {
            StaffClerkRelationDTO clerkRelationDTO = staffClerkRelationService.getByClerkId(clerkId);
            HmClerkRelationQDTO hmClerkRelationQDTO = new HmClerkRelationQDTO();
            hmClerkRelationQDTO.setRelationId(UniqueIdUtils.uniqueLong());
            hmClerkRelationQDTO.setHmId(hmId);
            hmClerkRelationQDTO.setWxUserId(clerkRelationDTO.getWxUserId());
            hmClerkRelationQDTO.setWxEnterpriseId(wxEnterpriseId);
            hmClerkRelationQDTO.setEnterpriseId(hmQrcodeQDTO.getEnterpriseId());
            hmClerkRelationQDTO.setStoreId(hmQrcodeQDTO.getStoreId());
            hmClerkRelationQDTO.setStoreName(hmQrcodeQDTO.getStoreName());
            hmClerkRelationQDTO.setStoreCode(hmQrcodeQDTO.getStoreCode());
            hmClerkRelationQDTO.setClerkId(clerkId);
            ClerkDTO clerkDTO = clerkService.getclerkById(clerkId);
            if (clerkDTO != null) {
                hmClerkRelationQDTO.setClerkName(clerkDTO.getClerkName());
                hmClerkRelationQDTO.setClerkCode(clerkDTO.getClerkCode());
            }
            clerkRelationQDTOS.add(hmClerkRelationQDTO);
        }
        hmClerkRelationService.insertBatch(clerkRelationQDTOS);
    }

    @Override
    public PageResult2<HmQrcodeListDTO> queryListByPage(HmQrcodeListQDTO hmQrcodeListQDTO) {
        return PageUtil.getPageInfo(hmQrcodeService.queryByPage(hmQrcodeListQDTO));
    }

    @Override
    public HmQrcodeDTO queryById(Long hmId) {
        return EntityUtil.changeEntityByOrika(HmQrcodeDTO.class, hmQrcodeService.queryById(hmId));
    }

    @Override
    public boolean delById(Long hmId) {
        HmQrcodeBO hmQrcodeBO = hmQrcodeService.queryById(hmId);
        if (hmQrcodeBO == null) {
            return false;
        }
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(hmQrcodeBO.getWxEnterpriseId());
        boolean flag = qywxUserApiService.delContactWay(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid(), hmQrcodeBO.getWxConfigId());
        if (flag) {
            hmQrcodeService.deleteById(hmId);
        }
        return true;
    }

    @Override
    public boolean delByClerkId(String clerkId, String enterpriseId) {
        List<HmQrcodeBO> hmQrcodeBOS = hmQrcodeService.listByClerkId(clerkId, enterpriseId);
        if (CollectionUtils.isEmpty(hmQrcodeBOS)) {
            logger.error("无活码：clerkId:{}",clerkId);
            return false;
        }

        for (HmQrcodeBO hmQrcodeBO : hmQrcodeBOS) {
            delById(hmQrcodeBO.getHmId());
        }
        return true;
    }
}
